using System;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace GlobalMasterPage {
	
	public class FeatureReceiver : SPFeatureReceiver 
	{
		const string defaultMasterUrl = "/_catalogs/masterpage/default.master";
		const string customizedMasterUrl = "/_catalogs/masterpage/MyCustom.master";
		const string previewImageName = "CustomizedMasterPreview.gif";
		
		public override void FeatureInstalled(SPFeatureReceiverProperties properties) 
		{
		}
		
		public override void FeatureUninstalling(SPFeatureReceiverProperties properties) 
		{
		}
		
		public override void FeatureActivated(SPFeatureReceiverProperties properties) 
		{
			SPSite site = properties.Feature.Parent as SPSite;
			if (site == null)
				return;
		
			SPWeb rootWeb = site.RootWeb;
			rootWeb.AllProperties["OldMasterUrl"] = rootWeb.MasterUrl;
			rootWeb.AllProperties["OldCustomMasterUrl"] = rootWeb.CustomMasterUrl;
			rootWeb.MasterUrl = customizedMasterUrl;
			rootWeb.CustomMasterUrl = customizedMasterUrl;
			rootWeb.Update();
			foreach (SPWeb subWeb in rootWeb.Webs)
			{
				ProcessSubWebs(subWeb, true);
			}
		}
		
		private void ProcessSubWebs(SPWeb web, bool isActivation)
		{
			if (isActivation)
			{
				web.AllProperties["OldMasterUrl"] = web.MasterUrl;
				web.AllProperties["OldCustomMasterUrl"] = web.CustomMasterUrl;
				web.MasterUrl = web.Site.RootWeb.MasterUrl;
				web.CustomMasterUrl = web.Site.RootWeb.MasterUrl;
			}
			else
			{
				DeactivateWeb(web);
			}
			web.Update();
			
			foreach (SPWeb subWeb in web.Webs) 
			{
				ProcessSubWebs(subWeb, isActivation);
			}
		}
		
		public override void FeatureDeactivating(SPFeatureReceiverProperties properties) 
		{
			SPSite site = properties.Feature.Parent as SPSite;
			if (site == null)
				return;
		
			SPWeb rootWeb = site.RootWeb;
			DeactivateWeb(rootWeb);
			rootWeb.Update();
			
			foreach (SPWeb subWeb in rootWeb.Webs)
			{
				ProcessSubWebs(subWeb, false);
			}

			if (rootWeb.MasterUrl != customizedMasterUrl)
			{
				try
				{
					bool fileExists = rootWeb.GetFile(customizedMasterUrl).Exists;
					SPFile file = rootWeb.GetFile(customizedMasterUrl);
					SPFolder masterPageGallery = file.ParentFolder;
					
					SPFolder temp = masterPageGallery.SubFolders.Add("Temp");
					file.MoveTo(temp.Url + "/" + file.Name);
					temp.Delete();

					fileExists = masterPageGallery.SubFolders["Preview Images"].SubFolders["Custom"].Exists;
					SPFolder customFolder = masterPageGallery.SubFolders["Preview Images"].SubFolders["Custom"];
					if (customFolder.Files.Count == 1 && customFolder.Files[0].Name == previewImageName)
					{
						masterPageGallery.SubFolders["Preview Images"].SubFolders["Custom"].Delete();
					}
					else if (customFolder.Files.Count > 1 && customFolder.Files[previewImageName].Exists)
					{
						customFolder.Files[previewImageName].Delete();
					}
				}
				catch (ArgumentException)
				{
					return;
				}
			}
		}

		private void DeactivateWeb(SPWeb web) {
			if (web.AllProperties.ContainsKey("OldMasterUrl")) {
				string oldMasterUrl = web.AllProperties["OldMasterUrl"].ToString();
				try {
					bool fileExists = web.GetFile(oldMasterUrl).Exists;
					web.MasterUrl = oldMasterUrl;
				}
				catch (ArgumentException) {
					web.MasterUrl = defaultMasterUrl;
				}

				string oldCustomUrl = web.AllProperties["OldCustomMasterUrl"].ToString();
				try {
					bool fileExists = web.GetFile(oldCustomUrl).Exists;
					web.CustomMasterUrl = web.AllProperties["OldCustomMasterUrl"].ToString();
				}
				catch (ArgumentException) {
					web.CustomMasterUrl = defaultMasterUrl;
				}

				web.AllProperties.Remove("OldMasterUrl");
				web.AllProperties.Remove("OldCustomMasterUrl");
			}
			else {
				web.MasterUrl = defaultMasterUrl;
				web.CustomMasterUrl = defaultMasterUrl;
			}
		}
	}
}
